import typing

def validate(request, schema, field, value, mode=None) -> typing.Optional[str]:
    """ Validate data. 

    Returns error message if fail.
    Returns None if no errors

    """
    return 



import morpfw
from dataclasses import dataclass, field
from datetime import date, datetime

import typing
import pytz

# {% if cookiecutter.project_type == "morpcc" %}
from morpcc.deform.referencewidget import ReferenceWidget
from morpcc.deform.vocabularywidget import VocabularyWidget
from morpcc.validator.reference import ReferenceValidator
from morpcc.validator.vocabulary import VocabularyValidator
# {% endif %}
from morpfw.validator.field import valid_identifier

@dataclass
class {{ cookiecutter.type_name }}Schema(morpfw.Schema):

    title: typing.Optional[str] = field(default=None, 
                                        metadata={'required': True})
    description: typing.Optional[str] = field(default=None,
                                        metadata={'format': 'text'})



import os
import sys
from functools import wraps

import click
import importscan
import yaml

import mfw_template


def validate_name(ctx, param, value):
    for c in value:
        if c in ['_']:
            continue
        if not c.isalnum():
            raise click.BadParameter("Name need to be alphanumeric")
    if not value[0].isalpha():
        raise click.BadParameter("Name must start with alphabet")
    return value


def to_identifier(text, lower=True):
    newtxt = ""
    if lower:
        text = text.lower()
    for c in text:
        if c == " ":
            newtxt += "_"
        elif c.isalnum():
            newtxt += c
    return newtxt


def require_config(f):
    def wrapper(ctx, *args, **kwargs):
        if not ctx.obj["RC"]["found_config"]:
            print("Please run this in project directory", file=sys.stderr)
            sys.exit(1)
        return f(ctx, *args, **kwargs)

    wrapper.__name__ = f.__name__
    return wrapper


@click.group()
@click.pass_context
def app(ctx=None):
    """Cookiecutter templates for Morp Framework"""
    localrc = os.path.join(os.getcwd(), ".mfwtemplaterc")
    c = {"found_config": False}
    if os.path.exists(localrc):
        with open(localrc, "r") as f:
            c = yaml.load(f, Loader=yaml.Loader)
            c["found_config"] = True

    ctx.ensure_object(dict)
    ctx.obj["RC"] = c


def cli():
    importscan.scan(mfw_template)
    app()



import typing

def validate(request, schema, data, mode=None, **kw) -> typing.Optional[dict]:
    """ Validate data 

    return None if valid
    return dictionary with following structure if invalid::

       {'field': '<field_name>',
        'message': '<error_message>'}
    """
    return None



from ..app import App
from .model import {{cookiecutter.type_name}}Collection
from .model import {{cookiecutter.type_name}}Model
from .schema import {{cookiecutter.type_name}}Schema
from .path import get_collection, get_model
# {% if cookiecutter.project_type == "morpcc" %}
from .modelui import {{cookiecutter.type_name}}CollectionUI
from .modelui import {{cookiecutter.type_name}}ModelUI
# {% endif %}


@App.typeinfo(
    name='{{ cookiecutter.project_name }}.{{ cookiecutter.module_name }}',
    schema={{cookiecutter.type_name}}Schema)
def get_typeinfo(request):
    return {
        'title': '{{ cookiecutter.type_name }}',
        'description': '{{ cookiecutter.type_description }}',
        'icon': 'database',
        'schema': {{cookiecutter.type_name}}Schema,
        'collection': {{cookiecutter.type_name}}Collection,
        'collection_factory': get_collection,
        'model': {{cookiecutter.type_name}}Model,
        'model_factory': get_model,
        # {% if cookiecutter.project_type == "morpcc" %}
        'collection_ui': {{cookiecutter.type_name}}CollectionUI,
        'model_ui': {{cookiecutter.type_name}}ModelUI,
        # {% endif %}
        'internal': False
    }



import os

import click
import yaml
from cookiecutter.main import cookiecutter
from cryptography.fernet import Fernet
from pkg_resources import resource_filename

from ..cli import app, require_config, to_identifier, validate_name


@app.command(help="create a new Field Validator")
@click.option("--name", prompt="Validator Name", callback=validate_name)
@click.pass_context
@require_config
def create_fieldvalidator(ctx, name):
    conf = ctx.obj["RC"].copy()
    conf["validator_name"] = name
    module_name = to_identifier(name)
    conf["module_name"] = module_name
    cookiecutter(
        resource_filename("mfw_template", "templates/fieldvalidator"),
        extra_context=conf,
        no_input=True,
        output_dir=conf["project_name"],
        overwrite_if_exists=True,
        skip_if_file_exists=True,
    )



import rulez

from morpcc.crud.view.edit import edit as default_edit
from morpcc.crud.view.listing import listing as default_listing
from morpcc.crud.view.view import view as default_view
from morpfw.crud import permission as crudperm

from ..app import App
from .model import {{cookiecutter.type_name}}Model, {{cookiecutter.type_name}}Collection
# {% if cookiecutter.project_type == "morpcc" %}
from .modelui import {{cookiecutter.type_name}}ModelUI, {{cookiecutter.type_name}}CollectionUI
# {% endif %}





import typing
from dataclasses import dataclass, field

from morpcc.behavior.base import BaseBehavior

from ..app import App
from .model import ({{ cookiecutter.behavior_name }}Collection,
                    {{ cookiecutter.behavior_name }}Model)
from .modelui import ({{ cookiecutter.behavior_name }}CollectionUI,
                      {{ cookiecutter.behavior_name }}ModelUI)
from .schema import {{ cookiecutter.behavior_name }}Schema


class {{ cookiecutter.behavior_name }}(BaseBehavior):
    """ Behavior component for {{ cookiecutter.behavior_name }}"""
    schema = {{ cookiecutter.behavior_name }}Schema
    model_marker = {{ cookiecutter.behavior_name }}Model
    modelui_marker = {{ cookiecutter.behavior_name }}ModelUI
    collection_marker = {{ cookiecutter.behavior_name }}Collection
    collectionui_marker = {{ cookiecutter.behavior_name }}CollectionUI


@App.behavior('{{ cookiecutter.project_name }}.{{ cookiecutter.module_name }}')
def get_behavior(request):
    """ Behavior factory for {{ cookiecutter.behavior_name }} """
    return {{ cookiecutter.behavior_name }}



import shutil
import os

project_name = '{{ cookiecutter.project_name }}'
project_type = '{{ cookiecutter.project_type }}'
module_name = '{{ cookiecutter.module_name }}'

if project_type != 'morpcc':
    os.unlink('modelui.py')

if project_type == 'morpcc':
    os.unlink('permission_rule.py')



import morpfw
import rulez
from datetime import datetime, date
import pytz

from .schema import {{cookiecutter.type_name}}Schema
# {% if cookiecutter.project_type == "morpcc" %}
from .modelui import {{cookiecutter.type_name}}ModelUI, {{cookiecutter.type_name}}CollectionUI
# {% endif %}

class {{cookiecutter.type_name}}Model(morpfw.Model):
    schema = {{cookiecutter.type_name}}Schema

# {% if cookiecutter.project_type == "morpcc" %}
    def ui(self):
        return {{cookiecutter.type_name}}ModelUI(self.request, self,
                self.collection.ui())
# {% endif %}


class {{cookiecutter.type_name}}Collection(morpfw.Collection):
    schema = {{cookiecutter.type_name}}Schema

# {% if cookiecutter.project_type == "morpcc" %}
    def ui(self):
        return {{cookiecutter.type_name}}CollectionUI(self.request, self)
# {% endif %}




import morpfw
from morpfw.authz.pas import DefaultAuthzPolicy
from morpfw.crud import permission as crudperm

# {% if cookiecutter.project_type == "morpfw" %}

# {% elif cookiecutter.project_type == "morpcc" %}
import morpcc
import morpcc.permission as ccperm

# {% endif %}

# {% if cookiecutter.project_type == "morpfw" %}


class AppRoot(object):
    """ Application root for {{ cookiecutter.project_name }}"""

    def __init__(self, request):
        self.request = request


class App(DefaultAuthzPolicy, morpfw.SQLApp):
    """ {{ cookiecutter.project_name }} Application """

    pass


@App.path(model=AppRoot, path="/")
def get_approot(request):
    return AppRoot(request)


@App.permission_rule(model=AppRoot, permission=crudperm.All)
def allow_all(identity, context, permission):
    """ Default permission rule, allow all """
    return True


# {% elif cookiecutter.project_type == "morpcc" %}


class AppRoot(morpcc.Root):
    """ Application root for {{ cookiecutter.project_name }}"""

    pass


class App(morpcc.App):
    """ {{ cookiecutter.project_name }} Application """

    pass


@App.path(model=AppRoot, path="/")
def get_approot(request):
    return AppRoot(request)


@App.template_directory()
def get_template_directory():
    return "templates"


# {% endif %}



import os
import sys

import click
import yaml
from cookiecutter.main import cookiecutter
from cryptography.fernet import Fernet
from pkg_resources import resource_filename

from ..cli import app, require_config, to_identifier


@app.command(help="create a new resource type")
@click.pass_context
@click.option("--name", prompt="Type Name")
@require_config
def create_resource(ctx, name):
    conf = ctx.obj["RC"].copy()
    conf["type_name"] = name
    proj_name = conf['project_name']
    module_name = to_identifier(name)
    module_name = click.prompt("Module Name", default=module_name)
    type_desc = click.prompt("Type Description", default="%s type" % name)
    api_mount_path = click.prompt("API Mount Path", 
            default="/api/v1/%s.%s" % (proj_name, module_name))
    ui_mount_path = click.prompt("UI Mount Path", 
            default="/%s.%s" % (proj_name, module_name))

    conf.update(
        {
            "module_name": module_name,
            "type_name": name,
            "type_description": type_desc,
            "api_mount_path": api_mount_path,
            "ui_mount_path": ui_mount_path,
        }
    )
    cookiecutter(
        resource_filename("mfw_template", "templates/resource"),
        extra_context=conf,
        no_input=True,
        output_dir=conf["project_name"],
    )



import morpfw
import morpfw.sql
from morpfw.sql import construct_orm_model
import sqlalchemy as sa
import sqlalchemy_utils as sautils
import sqlalchemy_jsonfield as sajson

from ..app import App
from .model import {{cookiecutter.type_name}}Model

class {{cookiecutter.type_name}}Storage(morpfw.SQLStorage):

    model = {{cookiecutter.type_name}}Model

    orm_model = construct_orm_model(
            schema={{cookiecutter.type_name}}Model.schema,
            metadata=morpfw.sql.Base.metadata,
            name='{{cookiecutter.project_name}}_{{cookiecutter.module_name}}'
    )

@App.storage(model={{cookiecutter.type_name}}Model)
def get_storage(model, request, blobstorage):
    return {{cookiecutter.type_name}}Storage(request, blobstorage=blobstorage)


@App.blobstorage(model={{cookiecutter.type_name}}Model)
def get_blobstorage(model, request):
    return request.app.get_config_blobstorage(request)



from ..app import App
from .model import {{cookiecutter.type_name}}Model, {{cookiecutter.type_name}}Collection
# {% if cookiecutter.project_type == "morpcc" %}
from .modelui import {{cookiecutter.type_name}}ModelUI, {{cookiecutter.type_name}}CollectionUI
# {% endif %}
from .storage import {{cookiecutter.type_name}}Storage


def get_collection(request):
    storage = request.app.get_storage({{cookiecutter.type_name}}Model, request)
    return {{cookiecutter.type_name}}Collection(request, storage)


def get_model(request, identifier):
    col = get_collection(request)
    return col.get(identifier)


@App.path(model={{cookiecutter.type_name}}Collection,
          path='{{ cookiecutter.api_mount_path }}')
def _get_collection(request):
    return get_collection(request)


@App.path(model={{cookiecutter.type_name}}Model,
          path='{{ cookiecutter.api_mount_path }}/{identifier}')
def _get_model(request, identifier):
    return get_model(request, identifier)

# {% if cookiecutter.project_type == "morpcc" %}


@App.path(model={{cookiecutter.type_name}}CollectionUI,
          path='{{ cookiecutter.ui_mount_path }}')
def _get_collection_ui(request):
    collection = get_collection(request)
    return collection.ui()


@App.path(model={{cookiecutter.type_name}}ModelUI,
          path='{{ cookiecutter.ui_mount_path }}/{identifier}')
def _get_model_ui(request, identifier):
    model = get_model(request, identifier)
    if model:
        return model.ui()

# {% endif %}



from morpcc.crud.model import ModelUI, CollectionUI


class {{cookiecutter.type_name}}ModelUI(ModelUI):
    pass


class {{cookiecutter.type_name}}CollectionUI(CollectionUI):
    modelui_class = {{cookiecutter.type_name}}ModelUI



from setuptools import setup, find_packages
import sys
import os

version = '{{ cookiecutter.version }}'

def readfile(name):
    with open(os.path.join(os.path.dirname(__file__), name)) as f:
        out = f.read()
    return out

desc = '\n\n'.join([readfile('README.rst'), readfile('CHANGELOG.rst')])

setup(name='{{ cookiecutter.project_name }}',
      version=version,
      description="{{ cookiecutter.short_description }}",
      long_description=desc,
      classifiers=[],  # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      keywords='',
      author='{{ cookiecutter.author_name }}',
      author_email='{{ cookiecutter.author_email }}',
      url='{{ cookiecutter.project_url }}',
      license='{{ cookiecutter.license }}',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      python_requires=">=3.7",
      install_requires=[
          # -*- Extra requirements: -*-
          'morpfw>=0.4.0b1',
          # {% if cookiecutter.project_type == "morpcc" %}
          'morpcc>=0.1.0a4'
          # {% endif %}
      ],
      extras_require={
          'test': [
              'morpfw[test]',
          ]
      },
      entry_points="""
      # -*- Entry points: -*-
      """,
      )





class {{ cookiecutter.behavior_name }}ModelUI(object):
    """ ModelUI marker for {{ cookiecutter.behavior_name }}"""
    pass

class {{ cookiecutter.behavior_name }}CollectionUI(object):
    """ Collection UI marker for {{ cookiecutter.behavior_name }}"""
    pass



import shutil
import os

project_name = '{{ cookiecutter.project_name }}'
project_type = '{{ cookiecutter.project_type }}'

if project_type != 'morpcc':
    shutil.rmtree('%s/templates' % project_name)
    shutil.rmtree('%s/static_files' % project_name)
    os.unlink('%s/static.py' % project_name)



import typing
from dataclasses import dataclass, field

from morpcc.applicationbehavior.base import BaseBehavior

from ..app import App
from .model import {{ cookiecutter.behavior_name }}Model
from .modelui import {{ cookiecutter.behavior_name }}ModelUI


class {{ cookiecutter.behavior_name }}(BaseBehavior):
    """
    Behavior component for {{ cookiecutter.behavior_name }}
    """
    model_marker = {{ cookiecutter.behavior_name }}Model
    modelui_marker = {{ cookiecutter.behavior_name }}ModelUI

    # you may specify entity behavior mapping here , which
    # will mark named entities in this application with 
    # a behavior marker
    # name -> BehaviorClass
    # * -> BehaviorClass (apply to all)
    entity_behavior = {}


@App.application_behavior("{{ cookiecutter.project_name }}.{{ cookiecutter.module_name }}")
def get_behavior(request):
    """Behavior marker factory for {{ cookiecutter.behavior_name }}"""
    return {{ cookiecutter.behavior_name }}





import os
from configparser import ConfigParser

import click
import yaml
from click.decorators import option
from click.termui import prompt
from cookiecutter.main import cookiecutter
from cryptography.fernet import Fernet
from pkg_resources import resource_filename
from datetime import datetime
from ..cli import app, cli, validate_name


def get_git_config():
    gitconfig = os.path.join(os.environ.get("HOME"), ".gitconfig")
    if not os.path.exists(gitconfig):
        return lambda section, option: None
    cf = ConfigParser()
    cf.readfp(open(gitconfig))

    def get_opt(section, option):
        if cf.has_option(section, option):
            return cf.get(section, option)
        return None

    return get_opt


git_conf = get_git_config()


@app.command(help="create a new MorpFW project")
@click.option("--name", prompt="Project Name", callback=validate_name)
@click.option(
    "--type",
    type=click.Choice(["morpcc", "morpfw"], case_sensitive=False),
    prompt="Project Type",
)
@click.option("--url", prompt="Project URL", default="http://myproject.github.io")
@click.option(
    "--author", prompt="Author Name", default=git_conf("user", "name"), required=True
)
@click.option(
    "--author-email",
    prompt="Author Email",
    default=git_conf("user", "email"),
    required=True,
)
@click.option(
    "--license",
    type=click.Choice(["Apache2", "GPLv3+", "AGPLv3+", "Proprietary"], case_sensitive=False),
    prompt="License",
)
def create_project(name, type, url, author, author_email, license):
    now = datetime.now()
    rpm_date = now.strftime('%a %b %d %Y')
    cookiecutter(
        resource_filename("mfw_template", "templates/project"),
        extra_context={
            "project_name": name,
            "project_type": type,
            "project_url": url,
            "project_license": license,
            "author_name": author,
            "author_email": author_email,
            "fernet_key": Fernet.generate_key().decode("utf-8"),
            "rpm_date": rpm_date,
        },
        no_input=True,
    )



from morpfw.static import StaticRoot as BaseStaticRoot
from .app import App


class StaticRoot(BaseStaticRoot):

    module = '{{ cookiecutter.project_name }}'
    directory = 'static_files'


@App.path(model=StaticRoot, path='/__static__/{{ cookiecutter.project_name }}', absorb=True)
def get_staticroot(absorb):
    return StaticRoot(absorb)




import os
import sys

from setuptools import find_packages, setup

version = "0.3b3"

def readfile(name):
    with open(os.path.join(os.path.dirname(__file__), name)) as f:
        out = f.read()
    return out

desc = '\n'.join([readfile('README.rst'), readfile('CHANGELOG.rst')])

setup(
    name="mfw-template",
    version=version,
    description="Cookiecutter templates for Morp Framework",
    long_description=desc,
    classifiers=[],  # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
    keywords="",
    author="Izhar Firdaus",
    author_email="kagesenshi.87@gmail.com",
    url="http://github.com/morpframework",
    license="MIT",
    packages=find_packages(exclude=["ez_setup", "examples", "tests"]),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "cookiecutter>=1.7.2",
        "click>=7.1.2",
        "importscan",
        "pyyaml>=4.2b1",
        "cryptography",
        # -*- Extra requirements: -*-
    ],
    extras_require={
        "docs": [
            'sphinx-click'
        ]
    },
    entry_points={
        "console_scripts": [
            "mfw-template=mfw_template.cli:cli",
        ]
    },
)



from morpfw.crud import permission as crudperm
from ..app import App
from .model import {{cookiecutter.type_name}}Model, {{cookiecutter.type_name}}Collection
from .modelui import {{cookiecutter.type_name}}ModelUI, {{cookiecutter.type_name}}CollectionUI


@App.permission_rule(model={{cookiecutter.type_name}}Collection,
                     permission=crudperm.All)
def allow_collection_access(identity, model, permission):
    return True

@App.permission_rule(model={{cookiecutter.type_name}}Model,
                     permission=crudperm.All)
def allow_model_access(identity, model, permission):
    return True




import os

import click
import yaml
from cookiecutter.main import cookiecutter
from cryptography.fernet import Fernet
from pkg_resources import resource_filename

from ..cli import app, require_config, to_identifier, validate_name


@app.command(help="create a new Entity Behavior")
@click.pass_context
@click.option("--name", prompt="Behavior Name", callback=validate_name)
@require_config
def create_behavior(ctx, name):
    conf = ctx.obj["RC"].copy()
    conf["behavior_name"] = name
    module_name = to_identifier(name)
    module_name = click.prompt("Module Name", default=module_name)
    conf["module_name"] = module_name
    cookiecutter(
        resource_filename("mfw_template", "templates/behavior"),
        extra_context=conf,
        no_input=True,
        output_dir=conf["project_name"],
    )




class {{ cookiecutter.behavior_name }}Model(object):
    """ Model marker for {{ cookiecutter.behavior_name }}"""
    pass


class {{ cookiecutter.behavior_name }}Collection(object):
    """ Collection marker for {{ cookiecutter.behavior_name }}"""
    pass

